/** 
 *  Yudit Unicode Editor Source File
 *
 *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
 *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
 *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2,
 *  dated June 1991. See file COPYYING for details.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SParagraph_h
#define SParagraph_h

#include "stoolkit/SGlyph.h"
#include "stoolkit/STypes.h"

class SParagraph 
{
public: 
  SParagraph(void);
  ~SParagraph();

  SParagraph(const SV_UCS4& buffer, unsigned int *index);
  SParagraph* subParagraph(unsigned int from, unsigned int to) const;

  inline unsigned int size() const;
  inline const SGlyph& operator [] (unsigned int index) const;
  inline const SGlyph* peek (unsigned int index) const;

  inline void append(const SGlyph& glyph);
  inline void insert(unsigned int into, const SGlyph& glyph);
  inline void remove(unsigned int from, unsigned int to);
  inline void remove(unsigned int at);
  inline void truncate(unsigned int to);
  inline void replace(unsigned int at, const SGlyph& glyph);
  inline void clear();

  void reShapeSegment ();
  void reShapeSegment (const SGlyph* before, const SGlyph* after);

  int getNonTransparentBefore (unsigned int index);
  int getNonTransparentAfter (unsigned int index);
  
  void toVisual ();
  void toLogical();

private:
  SVector<SGlyph> glyphs;
};

unsigned int
SParagraph::size() const
{
  return glyphs.size();
}

const SGlyph&
SParagraph::operator [] (unsigned int index) const
{
  return glyphs[index];
}

const SGlyph*
SParagraph::peek (unsigned int index) const
{
  return glyphs.peek(index);
}
void
SParagraph::clear()
{
  glyphs.clear();
}

void
SParagraph::insert(unsigned int into, const SGlyph& glyph)
{
  glyphs.insert (into, glyph);
}


void
SParagraph::append(const SGlyph& glyph)
{
  glyphs.append (glyph);
}

void
SParagraph::remove(unsigned int from, unsigned int to)
{
  glyphs.remove (from, to);
}

void
SParagraph::remove(unsigned int at)
{
  glyphs.remove (at);
}

void
SParagraph::truncate(unsigned int to)
{
  glyphs.truncate (to);
}

void
SParagraph::replace(unsigned int at, const SGlyph& glyph)
{
  glyphs.replace (at, glyph);
}



#endif /* SParagraph_h */
